



if not exist %PREFIX%\\Library\\include\\thrift\\Thrift.h exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if exist %PREFIX%\\Library\\bin\\thrift.exe exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Library\\lib\\thriftmd.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
